#
#
# Copyright (c)2004 DuckCorp(tm)
#
#
#
# This file is part of Coin-diff.
#
# Coin-diff is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Coin-diff is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Coin-diff; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#



require 'open-uri'
require 'zlib'
require 'bdb'
require 'pathname'
require 'observer'

#
# LINKS :
#   - manage BDB save/load :
#     + http://moulon.inra.fr/ruby/bdb.html
#     + http://www.rubydoc.org/book/c200.htm
#   - http://savannah.nongnu.org/cgi-bin/viewcvs/turtle/turtle/Turtle/Group/Debian/Package.pm?rev=1.64&content-type=text/vnd.viewcvs-markup
#




class Object
	
	def classtree()
		cl = self.class
		list = [ ]

		while cl != nil
			list<< cl
			cl = cl.superclass
		end

		return list
	end

end

class String

	def common(str)
		strz = ""

		i = 0
		while (i < str.length) and (i< self.length) and (self[i] == str[i])
			strz+= str[i,1]
			i+= 1
		end

		return str
	end

	def clean_url()
		url_parts = self.split("://");
		clean_path = Pathname.new(url_parts[1]).cleanpath.to_s
		return url_parts[0] + "://" + clean_path
	end

end

class Array

	def common_string()
		tab = Array.new(self)
		strz = tab.shift

		tab.each do |str|
			strz = str.common(strz)
			break if strz == ""
		end

		return strz
	end

	def match_regex?(regex)
		ok = true

		self.each do |el|
			if not el =~ regex
				ok = false
				break
			end
		end

		return ok
	end

end



module CoinDiff

	class Pkg
		attr_reader :package, :version, :maintainer, :architecture, :repository, :obsolete, :type
		attr_writer :package, :version, :maintainer, :architecture, :repository, :obsolete

		include Observable

		def initialize()
			clear
		end

		def clear()
			@type = @package = @version = @maintainer = @repository = ""
			@architecture = []
		end

		def initialize(hash)
			clear

			@package = hash['package']
			@version = hash['version']
			@maintainer = hash['maintainer']
			@architecture = []
			# architecture : managed in child classes
			@repository = hash['repository']
			@obsolete = false
		end

		def obsolete=(obs)
			changed if obs != @obsolete
			@obsolete = obs
			notify_observers(self)
		end

		def Pkg.get_field_list()
			return ["Package", "Version", "Maintainer", "Architecture"]
		end

		def Pkg::compare_ver_nobinnmu(ver1, ver2)
			i1 = ver1.rindex("-")
			if i1
				ver1_upstream = ver1[0..i1-1]
				ver1_debian = ver1[i1+1..-1]
				ver1_debian_parts = ver1_debian.split('.')
				ver1 = ver1_upstream + "-" + ver1_debian_parts[0]
				ver1 += "." + ver1_debian_parts[1] if ver1_debian_parts[1] and ver1_debian_parts[1] != "0"
			end
			i2 = ver2.rindex("-")
			if i2
				ver2_upstream = ver2[0..i2-1]
				ver2_debian = ver2[i2+1..-1]
				ver2_debian_parts = ver2_debian.split('.')
				ver2 = ver2_upstream + "-" + ver2_debian_parts[0]
				ver2 += "." + ver2_debian_parts[1] if ver2_debian_parts[1] and ver2_debian_parts[1] != "0"
			end
			return ver1 <=> ver2
		end

		# this is a pure virtual class
		private_class_method :new
	end

	class PkgBin < Pkg
		attr_reader :source, :priority, :section, :subsection, :depends, :suggests, :recommends, :conflicts, :provides, :replaces, :filename, :size, :installed_size, :md5sum, :short_description, :description, :real_version
		attr_writer :source, :priority, :section, :subsection, :depends, :suggests, :recommends, :conflicts, :provides, :replaces, :filename, :size, :installed_size, :md5sum, :short_description, :description, :real_version

		def initialize()
			clear
		end

		def clear()
			super

			@type = "bin"
			@source = @priority = @section = @subsection = @filename = @md5sum = @short_description = @description = @real_version = ""
			@size = @installed_size = 0
			@depends = @suggests = @recommends = @conflicts = @provides = @replaces = []
		end

		def initialize(hash)
			clear
			super(hash)
			
			@architecture = hash['architecture']

			@source = hash['source']
			@priority = hash['priority'] if hash['priority']
			fullsection = hash['section'].split("/")
			if fullsection.length == 1
				@section = "main"
				@subsection = hash['section']
			else
				@section = fullsection[0]
				@subsection = fullsection[1]
			end
			@depends = hash['depends'].split(", ") if hash['depends']
			@suggests = hash['suggests'].split(", ") if hash['suggests']
			@recommends = hash['recommends'].split(", ") if hash['recommends']
			@conflicts = hash['conflicts'].split(", ") if hash['conflicts']
			@replaces = hash['replaces'].split(", ") if hash['replaces']
			@provides = hash['provides'].split(", ") if hash['provides']
			@filename = hash['filename']
			@size = hash['size']
			@installed_size = hash['installed-size'] if hash['installed-size']
			@md5sum = hash['md5sum']
			@short_description = hash['short-description']
			@description = hash['description']
			if hash['real_version']
				@real_version = hash['real_version']
			else
				@real_version = @version
			end
		end

		def PkgBin.get_field_list()
			base_list = Pkg.get_field_list()
			return base_list+["Source", "Priority", "Section",  "Depends", "Suggests", "Recommends", "Conflicts", "Provides", "Replaces", "Filename", "Size", "Installed-Size", "MD5sum", "Description"]
		end

		def PkgBin.priority_lvl
			priority_lvl = {
				"required"	=> 40,
				"important"	=> 20,
				"standard"	=> 0,
				"optional"	=> -20,
				"extra"		=> -40
			}
			return priority_lvl
		end

		def PkgBin.section_lvl
			section_lvl = {
				"main"			=> 0,
				"contrib"		=> -10,
				"non-free"		=> -30
			}
			return section_lvl
		end

		def PkgBin.subsection_lvl
			subsection_lvl = {
				"libs"			=> 30,
				"base"			=> 20,
				"debian-installer"	=> 15,
				"devel"			=> 10,
				"interpreters"		=> 10,
				"admin"			=> 0,
				"doc"			=> 0,
				"mail"			=> 0,
				"net"			=> 0,
				"editors"		=> 0,
				"shells"		=> 0,
				"utils"			=> 0,
				"text"			=> 0,
				"electronics"		=> -10,
				"graphics"		=> -10,
				"hamradio"		=> -10,
				"comm"			=> -10,
				"math"			=> -10,
				"news"			=> -10,
				"sound"			=> -10,
				"tex"			=> -10,
				"web"			=> -10,
				"x11"			=> -10,
				"misc"			=> -20,
				"otherosfs"		=> -20,
				"games"			=> -20,
				"oldlibs"		=> -30
			}
			return subsection_lvl
		end

		def realprio
			return priority_lvl[priority]+section_lvl[section]+section_lvl[subsection]
		end

		def to_s
			str = ""

			str+= "Package: "+@package
			str+= " /!\\ OBSOLETE /!\\" if @obsolete
			str+= "\n"
			str+= "  - Source: "+@source+"\n"
			str+= "  - Priority: "+@priority+"\n"
			str+= "  - Section: "+@section+"\n"
			str+= "  - Subsection: "+@subsection+"\n"
			str+= "  - Maintainer: "+@maintainer+"\n"
			str+= "  - Architecture: "+@architecture+"\n"
			if @real_version != @version
				str+= "  - Version: "+@real_version+" (built from source version : "+@version+")\n"
			else
				str+= "  - Version: "+@version+"\n"
			end
			str+= "  - Provides: "+"\n"
			@provides.each do |bldd|
				str+= "    + "+bldd+"\n"
			end
			str+= "  - Depends: "+"\n"
			@depends.each do |bldd|
				str+= "    + "+bldd+"\n"
			end
			str+= "  - Conflits: "+"\n"
			@conflicts.each do |bldd|
				str+= "    + "+bldd+"\n"
			end
			str+= "  - Replaces: "+"\n"
			@replaces.each do |bldd|
				str+= "    + "+bldd+"\n"
			end
			str+= "  - Recommends: "+"\n"
			@recommends.each do |bldd|
				str+= "    + "+bldd+"\n"
			end
			str+= "  - Suggests: "+"\n"
			@suggests.each do |bldd|
				str+= "    + "+bldd+"\n"
			end
			str+= "  - Repository: "+@repository+"\n"
			str+= "  - Filename: "+@filename+"\n"
			str+= "  - MD5sum: "+@md5sum+"\n"
			str+= "  - Size: "+@size.to_s+"\n"
			str+= "  - Installed-Size: "+@installed_size.to_s+"\n"
			str+= "  - Short-Description: "+@short_description+"\n"
			str+= "  - Description: "+@description+"\n"

			return str
		end

		public_class_method :new
	end

	class PkgSrc < Pkg
		attr_reader :format, :standard_version, :directory, :binary
		attr_writer :format, :standard_version, :directory, :binary

		def initialize()
			clear
		end

		def clear()
			super

			@type = "src"
			@format = @standard_version = @directory = ""
			@binary = @build_depends = @uploaders = @files = @architecture = []
		end

		def initialize(hash)
			clear
			super(hash)
			
			@architecture = hash['architecture'].split(' ') if hash['architecture']

			@format = hash['format'] if hash['format']
			@standard_version = hash['standards-version'] if hash['standards-version']
			@directory = hash['directory']
			@binary = hash['binary'].split(", ") if hash['binary']
			@build_depends = hash['build-depends'].split(", ") if hash['build-depends']
			@uploaders = hash['uploaders'].split(", ") if hash['uploaders']
			@files = hash['files'].split("\n") if hash['files']
		end

		def PkgSrc.get_field_list()
			base_list = Pkg.get_field_list()
			return base_list+["Format", "Binary",  "Standards-Version", "Build-Depends", "Uploaders", "Directory", "Files"]
		end

		def to_s
			str = ""

			str+= "Package: "+@package
			str+= " /!\\ OBSOLETE /!\\" if @obsolete
			str+= "\n"
			str+= "  - Format: "+@format+"\n"
			str+= "  - Standards-Version: "+@standard_version+"\n"
			str+= "  - Maintainer: "+@maintainer+"\n"
			str+= "  - Uploaders: "+"\n"
			@uploaders.each do |upl|
				str+= "    + "+upl+"\n"
			end
			str+= "  - Architecture: "+"\n"
			@architecture.each do |arch|
				str+= "    + "+arch+"\n"
			end
			str+= "  - Version: "+@version+"\n"
			str+= "  - Binary: "+"\n"
			@binary.each do |bin|
				str+= "    + "+bin+"\n"
			end
			str+= "  - Build-Depends: "+"\n"
			@build_depends.each do |bldd|
				str+= "    + "+bldd+"\n"
			end
			str+= "  - Repository: "+@repository+"\n"
			str+= "  - Directory: "+@directory+"\n"
			str+= "  - Files: "+"\n"
			@files.each do |f|
				str+= "    + "+f+"\n"
			end

			return str
		end

		def arch_available?(arch)
			return true if @architecture.include?("all") or @architecture.include?("any")
			return @architecture.include?(arch)
		end

		public_class_method :new
	end

	class PkgSet
		attr_reader :name, :obsolete
		attr_writer :obsolete

		def initialize(pkg)
			@name = pkg
			@src_pkg_list = []
			@bin_pkg_list = {}
			@missing_pkg_list = {}
		end

		def obsolete=(obs)
			@obsolete = obs
			@src_pkg_list.each do |pkg|
				pkg.obsolete = obs
			end
		end

		def update(obj)
			if obj.classtree.include?(CoinDiff::Pkg)
				obj.obsolete = @obsolete if obj.obsolete != @obsolete
			end
		end

		def <<(pkg)
			if pkg.type == "src"
				if pkg.package != @name
					STDERR.puts "src: "+@name+" / "+pkg.package
					return false
				end
				@src_pkg_list<< pkg
			else
				# removed to allow versionned src fix
				#if pkg.source != @name
				#	STDERR.puts "bin: "+@name+" / "+pkg.package+" ("+pkg.source+")"
				#	return false
				#end
				arch = pkg.architecture
				@bin_pkg_list[arch] = {} if not @bin_pkg_list.has_key?(arch)
				@bin_pkg_list[arch][pkg.package] = [] if not @bin_pkg_list[arch].has_key?(pkg.package)
				@bin_pkg_list[arch][pkg.package]<< pkg
			end

			return true
		end

		def get_lastest_src_version()
			lastest = nil
			@src_pkg_list.each do |pkg|
				lastest = pkg.version if (not lastest) or (pkg.version > lastest)
			end

			return lastest
		end

		def get_bin_list(arch)
			return [] if not arch_available?(arch)
			return @bin_pkg_list[arch].keys
		end

		def get_bin_versions(arch, bin)
			list = []

			if bin_archall?(bin)
				@bin_pkg_list['all'][bin].each do |pkg|
					list<< pkg.version
				end
			elsif bin_available_for_arch?(arch, bin)
				@bin_pkg_list[arch][bin].each do |pkg|
					list<< pkg.version
				end
			end

			return list
		end

		def grab_bin_from_pkgset(pkgset, arch_list, bin)
			arch_list.each do |arch|
				pkgset.get_bin_versions(arch, bin).each do |ver|
					self<< pkgset.get_bin(arch, bin, ver) if not bin_available?(arch, bin, ver)
				end
			end
		end

		def get_src(ver=nil)
			ver = get_lastest_src_version() if not ver
			return nil if not ver
			
			@src_pkg_list.each do |pkg|
				return pkg if (pkg.version == ver)
			end

			return nil
		end

		def get_lastest_bin_version(arch, bin)
			lastest = nil
			if bin_archall?(bin)
				@bin_pkg_list['all'][bin].each do |pkg|
					lastest = pkg.version if (not lastest) or (pkg.version > lastest)
				end
			elsif bin_available_for_arch?(arch, bin)
				@bin_pkg_list[arch][bin].each do |pkg|
					lastest = pkg.version if (not lastest) or (pkg.version > lastest)
				end
			end

			return lastest
		end

		def get_bin(arch, bin, ver=nil)
			ver = get_lastest_bin_version(arch, bin) if not ver
			return nil if not ver
			
			if bin_archall?(bin)
				@bin_pkg_list['all'][bin].each do |pkg|
					return pkg if (pkg.version == ver)
				end
			elsif bin_available_for_arch?(arch, bin)
				@bin_pkg_list[arch][bin].each do |pkg|
					return pkg if (pkg.version == ver)
				end
			end

			return nil
		end

		def get_priority(arch)
			ver = get_lastest_src_version() if not ver
			return nil if not ver

			priority_lvl = PkgBin.priority_lvl
			prio = nil

			# todo : get prio from binaries if no source pkg
			
			src = get_src(ver)
			src.binary.each do |bin|
				binver = get_lastest_bin_version(arch, bin)
				pkg = get_bin(arch, bin, binver) if binver
				next if pkg and pkg.obsolete
				prio = pkg.priority if pkg and ((not prio) or (priority_lvl[pkg.priority] > priority_lvl[prio]))
			end

			return prio
		end

		def get_pkgset_priority()
			priority_lvl = PkgBin.priority_lvl
			prio = nil

			get_arch_list.each do |arch|
				prio0 = get_priority(arch)
				prio = prio0 if prio0 and ((not prio) or (priority_lvl[prio0] > priority_lvl[prio]))
			end

			return prio
		end

		def <=>(pkgset)
			priority_lvl = PkgBin.priority_lvl
			
			prio0 = get_pkgset_priority()
			prio1 = pkgset.get_pkgset_priority()
			return @name <=> pkgset.name if prio0 == prio1
			return 1 if (priority_lvl[prio0] > priority_lvl[prio1])
			return -1
		end

		def bin_available?(arch, bin, ver=nil, strict=true)
			ver = get_lastest_bin_version(arch, bin) if not ver
			return false if not ver

			if bin_archall?(bin)
				@bin_pkg_list['all'][bin].each do |pkg|
					if strict
						return true if (pkg.version == ver)
					else
						return true if Pkg::compare_ver_nobinnmu(pkg.version, ver) == 0
					end
				end
			elsif bin_available_for_arch?(arch, bin)
				@bin_pkg_list[arch][bin].each do |pkg|
					if strict
						return true if (pkg.version == ver)
					else
						return true if Pkg::compare_ver_nobinnmu(pkg.version, ver) == 0
					end
				end
			end

			return false
		end

		def buildable_for_arch?(arch, ver=nil)
			pkg = get_src(ver)
			return false if not pkg
			return pkg.arch_available?(arch)
		end

		def get_not_uptodate(arch, ver=nil)
			return [] if missing?(arch)

			ver = get_lastest_src_version() if not ver
			return [] if not ver
			
			return [] if @obsolete or not buildable_for_arch?(arch, ver)

			src = get_src(ver)

			list = []
			src.binary.each do |bin|
				# todo : next if binary obsolescence
				list<< bin if not bin_available?(arch, bin, ver, false)
			end

			return list
		end

		def uncomplete?(arch)
			return (get_missing_binaries(arch).empty?)?false:true
		end

		def missing?(arch)
			src = get_src()
			return false if not src
			return (get_missing_binaries(arch).length == src.binary.length)?true:false
		end

		def get_missing_binaries(arch)
			compute_missing(arch) if not @missing_pkg_list.has_key?(arch)
			return @missing_pkg_list[arch]
		end

		def compute_missing(arch)
			@missing_pkg_list[arch] = []
			return if @obsolete or not buildable_for_arch?(arch)

			# check each src pkg
			list = []
			ver_list = get_src_version_list()
			ver_list.each do |ver|
				src = get_src(ver)
				list = []
				src.binary.each do |bin|
					# todo : next if binary obsolescence
					pkg = get_bin(arch, bin)
					if not pkg
						list<< bin
					elsif ["libs", "oldlibs"].include?(pkg.subsection)
						# because a bin pkg can be generated by several versionned src pkg
						#  and libs may be only available in the lastest version
						#  (ex: gcc-3.x and libgcc1)
						list<< bin if pkg.version < ver
					end
				end
				return if list.empty?
			end

			# check if binaries can fulfill the needs even if their src pkg is missing
			# (needed bin list is taken from the lastest src pkg)
			src = get_src()
			get_bin_version_list().each do |ver|
				list1 = []
				get_bin_list(arch).each do |bin|
					list1<< bin if bin_available?(arch, bin, ver)
				end
				list1.sort!
				list2 = src.binary.sort
				return if list1 == list2
			end

			@missing_pkg_list[arch] = list
		end

		def get_src_list()
			return @src_pkg_list
		end

		def get_src_count()
			return @src_pkg_list.length
		end

		def get_bin_ver_list(arch, bin)
			return [] if not arch_available?(arch)
			return [] if not bin_available_for_arch?(arch, bin)

			list = []
			@bin_pkg_list[arch][bin].each do |pkg|
				list<< pkg.version
			end

			return list
		end

		def get_bin_list_for_arch(arch)
			if arch == any
				list = []
				@bin_pkg_list.each do |archz|
					list = list | @bin_pkg_list[archz]
				end
				return list
			end
			return (@bin_pkg_list.has_key?(arch))?@bin_pkg_list[arch]:[]
		end

		def get_bin_list_for_ver(ver)
			list = []
			@bin_pkg_list.each_key do |arch|
				@bin_pkg_list[arch].each_key do |bin|
					@bin_pkg_list[arch][bin].each do |pkg|
						list<< bin if pkg.version == ver
					end
				end
			end
			list.uniq!
			return list
		end

		def get_bin_count(arch)
			if arch == any
				count = 0
				@bin_pkg_list.each do |archz|
					count+= @bin_pkg_list[archz].length
				end
				return count
			end
			return (@bin_pkg_list.has_key?(arch))?(@bin_pkg_list[arch].length):0
		end

		def get_src_version_list()
			list = []

			@src_pkg_list.each do |pkg|
				list<< pkg.version
			end

			list.uniq!
			return list
		end

		def get_bin_version_list()
			list = []
			
			@bin_pkg_list.each_key do |arch|
				@bin_pkg_list[arch].each_key do |bin|
					@bin_pkg_list[arch][bin].each do |pkg|
						list<< pkg.version
					end
				end
			end

			list.uniq!
			return list
		end

		def get_arch_list()
			return @bin_pkg_list.keys
		end

		def arch_available?(arch)
			return @bin_pkg_list.has_key?(arch)
		end

		def bin_available_for_arch?(arch, bin)
			return true if arch_available?(arch) and @bin_pkg_list[arch].has_key?(bin)
			return bin_archall?(bin)
		end

		def bin_archall?(bin)
			return false if not arch_available?('all')
			return @bin_pkg_list['all'].has_key?(bin)
		end

		def to_s
			str = ""
			bin_list = []

			str+= "PkgSet: "+@name+"\n"
			str+= "  - Sources: \n"
			src_ver_list = get_src_version_list.sort!
			src_ver_list.each do |ver|
				str+= "    + "+ver+"\n"
			end
			bin_ver_list = get_bin_version_list.sort!
			bin_ver_list.each do |ver|
				str+= "  - Binaries ("+ver+"): \n"
				get_bin_list_for_ver(ver).each do |bin|
					bin_list<< bin
					str+= "    + "+bin+":\n"
					if bin_archall?(bin)
						pkg = get_bin("all", bin, ver)
						if pkg
							if pkg.version != pkg.real_version
								str+= "      * all (built with version : "+pkg.real_version+")\n"
							else
								str+= "      * all\n"
							end
						end
					else
						get_arch_list.each do |arch|
							pkg = get_bin(arch, bin, ver)
							if pkg
								if pkg.version != pkg.real_version
									str+= "      * "+arch+" (built with version : "+pkg.real_version+")\n"
								else
									str+= "      * "+arch+"\n"
								end
							end
						end
					end
				end
			end

			str+= "  - Missing binaries (in all selected architectures - according to last src): \n"
			src = get_src()
			if src
				missing_bin_list = src.binary - bin_list
				missing_bin_list.each do |bin|
					str+= "    + "+bin+"\n"
				end
			end

			return str
		end

	end

	class PkgBrowser

		def initialize()
			@repo_list_src = {}
			@repo_list_bin = {}
			@pkgset_list = {}
			@bin_to_src_list = {}
			@bin_to_src_list_notrust = {}
			@ignore_list = {}
			@ignore_list_match = {}
			@ignore_list_bad = {}
			@has_udeb = false
		end

		def <<(pkgset)
			@pkgset_list[pkgset.name] = pkgset
			# TODO : refeed @bin_to_src_list & @bin_to_src_list_notrust
		end

		def check_obsolescence()
			arch_list = get_arch_list()
			list = []

			# source obsolescence
			@pkgset_list.each_pair do |name, set|
				#str = ""
				if set.obsolete
					list<< name
				else
					src = set.get_src
					next if not src
					nb = 0
					src.binary.each do |bin|
						other_src = (bin_to_src(bin) and (not bin_to_src(bin).include?(name)))
						archall_missing = ((not bin_to_src(bin)) and set.bin_archall?(bin))
						arch_missing = ((not bin_to_src(bin)) and set.buildable_for_arch?(bin))
						if other_src or archall_missing or arch_missing
							#str2 = ""
							#str2 = " (!!!)" if (bin_to_src(bin) and (not bin_to_src(bin).include?(name)))
							#str+= "  - missing '"+bin+"'"+str2+"\n"
							nb+= 1
						else
							break
						end
					end
					if (src.binary.length == 0) or (nb == src.binary.length)
						set.obsolete = true
						list<< name if not list.include?(name)
					end
				end
			end

			# binary obsolescence
			#
			# not done

			return list
		end

		def feed_bin_arch(repo_tag, url)
			field_list = PkgBin.get_field_list.collect! {|str| str.downcase}
			arch = nil
			begin
				filename = (url + "/Packages.gz").clean_url()
				open(filename) do |f|
					pkg = {}
					bin = src = nil
					ldesc = ""
					src_version = nil
					descmode = false
					gzreader = Zlib::GzipReader.new(f)
					gzreader.each_line do |line|
						line.chomp!
						if descmode
							if line[0..0] == " "
								if line == " ."
									ldesc<< "\n"
								else
									ldesc<< line
								end
							else
								pkg['description'] = ldesc
								descmode = false
							end
						end

						if not descmode
							if line == ""
								if bin and pkg.has_key?('section')
									pkg['package'] = bin
									pkg['source'] = src
									pkg['repository'] = repo_tag
									if src_version
										pkg['real_version'] = pkg['version']
										pkg['version'] = src_version
									end
									if bin[-5..-1] == "-udeb"
										@has_udeb = true
									end
									pkgobj = PkgBin.new(pkg)
									@pkgset_list[src] = PkgSet.new(src) if not @pkgset_list.has_key?(src)
									@pkgset_list[src]<< pkgobj
									if not @bin_to_src_list.has_key?(bin)
										@bin_to_src_list[bin] = [ src ]
									else
										@bin_to_src_list[bin] = @bin_to_src_list[bin] | [ src ]
									end
								elsif bin
									STDERR.puts "Problem : bin entry missing section ("+bin+") !!! ["+repo_tag+"]"
								elsif not pkg.empty?
									STDERR.puts "Problem : bad bin entry !!! ["+repo_tag+"]"
								end

								pkg = {}
								bin = src = nil
								ldesc = ""
								src_version = nil
							else
								line_parts = line.split(": ", 2)
								field = line_parts[0].downcase
								val = line_parts[1]

								if field == "package"
									src = bin = val
								elsif field == "source"
									tmp = val.split(" ", 2)
									src = tmp[0]
									src_version = tmp[1][1..-2] if tmp[1]
								elsif field_list.include?(field)
									if field == "description"
										field = "short-description"
										descmode = true
									elsif (field == "architecture") and (val != "all")
										arch = val
									end
									pkg[field] = val
								end
							end
						end
					end
				end

				return true if not arch
				@repo_list_bin[arch] = {} if not @repo_list_bin.has_key?(arch)
				@repo_list_bin[arch][repo_tag] = url
			rescue Timeout::Error, RuntimeError
				STDERR.puts "Problems fetching '"+filename+"': "+$!
				return false
			end

			return true
		end

		def feed_src(repo_tag, url)
			field_list = PkgSrc.get_field_list.collect! {|str| str.downcase}
			begin
				filename = (url+"/Sources.gz").clean_url()
				open(filename) do |f|
					pkg = {}
					src = nil
					files = ""
					filesmode = false
					gzreader = Zlib::GzipReader.new(f)
					gzreader.each_line do |line|
						line.chomp!
						if filesmode
							if line[0..0] == " "
								if line == " ."
									ldesc<< "\n"
								else
									ldesc<< line
								end
							else
								pkg['description'] = ldesc
								filesmode = false
							end
						end

						if not filesmode
							if line == ""
								if src
									pkg['package'] = src
									pkg['repository'] = repo_tag
									pkgobj = PkgSrc.new(pkg)
									@pkgset_list[src] = PkgSet.new(src) if not @pkgset_list.has_key?(src)
									@pkgset_list[src]<< pkgobj
									if pkg['Binary']
										list = pkg['binary'].split(", ")
										list.each do |bin|
											if not @bin_to_src_list_notrust.has_key?(bin)
												@bin_to_src_list_notrust[bin] = [ src ]
											else
												@bin_to_src_list_notrust[bin] = @bin_to_src_list_notrust[bin] | [ src ]
											end
										end
									end
								elsif not pkg.empty?
									STDERR.puts "Problem : bad src entry !!! ["+repo_tag+"]"
								end

								src = nil
								files = ""
								pkg = {}
							else
								line_parts = line.split(": ", 2)
								field = line_parts[0].downcase
								val = line_parts[1]

								if field == "package"
									src = val
								elsif field_list.include?(field)
									if field == "files"
										filesmode = true
									else
										pkg[field] = val
									end
								end
							end
						end
					end
				end

				@repo_list_src[repo_tag] = url
			rescue Timeout::Error, RuntimeError

				STDERR.puts "Problems fetching '"+filename+"': "+$!
				return false
			end

			return true
		end

		def get_conflicting_src_claims()
			list = {}

			@bin_to_src_list_notrust.each_pair do |bin, srclist|
				next if srclist.length<2
				srclist2 = []
				srclist.each do |src|
					srclist2<< src if not get_pkgset(src).obsolete
				end
				list[bin] = srclist2 if srclist2.length>=2
			end

			return list
		end

		def compute_versionned_src()
			claims_list = get_conflicting_src_claims()
			arch_list = get_arch_list()

			claims_list.each_pair do |bin, srclist|
				# avoid computing when all is already OK
				tab1 = bin_to_src(bin)
				tab1.sort! if tab1
				tab2 = srclist
				tab2.sort! if tab2
				next if tab1 == tab2

				# find common prefix
				com = srclist.common_string
				# check if versionned src name
				fmt_ok = srclist.match_regex?(/^.+-\d\.\d.*$/)
				if (com.length > 3) and fmt_ok
					binsrclist = bin_to_src(bin)
					next if not binsrclist

					binsrclist.each do |binsrc|
						srclist.each do |src|
							next if src == binsrc
							#puts bin.to_s+" : "+src.to_s+" <-- "+binsrc.to_s
							@pkgset_list[src].grab_bin_from_pkgset(@pkgset_list[binsrc], arch_list, bin)
						end
					end
					@bin_to_src_list[bin] = srclist
				end
			end
		end

		def feed_using_repolist_file(filename, is_arch=false)
			regex = /^"(.*)"[ \t]+((?:ftp|http):\/\/[^ ]+)$/i
			list = load_simple_list_from_file(filename, regex)
			list2 = {}
			list.each do |repo|
				repo =~ regex
				list2[$1] = $2
			end
			feed_using_repolist_array(list2, is_arch)
		end

		def feed_using_repolist_list(list, is_arch=false)
			list.each_pair do |repo_name, repo_loc|
				if is_arch
					feed_bin_arch(repo_name, repo_loc)
				else
					feed_src(repo_name, repo_loc)
				end
			end
		end

		def load_pkg_db(filename)
			begin
				db = BDB::Hash.open(filename, "pkginfos", "r", 0644)
			rescue
				STDERR.puts "Problem : cannot load pkg database file"
				return false
			end
			@repo_list_src = Marshal.restore(db['repo_list_src']) if db.has_key?('repo_list_src')
			@repo_list_bin = Marshal.restore(db['repo_list_bin']) if db.has_key?('repo_list_bin')
			@pkgset_list = Marshal.restore(db['pkgset_list']) if db.has_key?('pkgset_list')
			@bin_to_src_list = Marshal.restore(db['bin_to_src_list']) if db.has_key?('bin_to_src_list')
			@bin_to_src_list_notrust = Marshal.restore(db['bin_to_src_list_notrust']) if db.has_key?('bin_to_src_list_notrust')
			@ignore_list = Marshal.restore(db['ignore_list']) if db.has_key?('ignore_list')
			@ignore_list_match = Marshal.restore(db['ignore_list_match']) if db.has_key?('ignore_list_match')
			@ignore_list_bad = Marshal.restore(db['ignore_list_bad']) if db.has_key?('ignore_list_bad')
			db.close
			return true
		end

		def save_pkg_db(filename)
			begin
				db = BDB::Hash.open(filename, "pkginfos", "w", 0644)
			rescue
				STDERR.puts "Problem : cannot save pkg database file"
				return false
			end
			db['repo_list_src'] = Marshal.dump(@repo_list_src)
			db['repo_list_bin'] = Marshal.dump(@repo_list_bin)
			db['pkgset_list'] = Marshal.dump(@pkgset_list, -1)
			db['bin_to_src_list'] = Marshal.dump(@bin_to_src_list)
			db['bin_to_src_list_notrust'] = Marshal.dump(@bin_to_src_list_notrust)
			db['ignore_list'] = Marshal.dump(@ignore_list)
			db['ignore_list_match'] = Marshal.dump(@ignore_list_match)
			db['ignore_list_bad'] = Marshal.dump(@ignore_list_bad)
			db.close
			return true
		end

		def feed_arch_ignore(arch, list)
			@ignore_list[arch] = [] if not @ignore_list.has_key?(arch)
			@ignore_list[arch] = (@ignore_list[arch] | list)
			@ignore_list_match.delete(arch)
			@ignore_list_bad.delete(arch)
		end

		def feed_arch_ignore_from_file(arch, filename)
			regex = /^[^ \t]+$/
			list = load_simple_list_from_file(filename, regex)
			return feed_arch_ignore(arch, list)
		end

		def get_bin_arch_repo_list(arch)
			return (@repo_list_bin.has_key?(arch))?@repo_list_bin[arch]:[]
		end

		def get_src_repo_list()
			return @repo_list_src
		end

		def get_pkgset(pkg)
			return @pkgset_list[pkg]
		end

		def bin_to_src(pkg)
			return @bin_to_src_list[pkg]
		end

		def get_arch_list()
			return @repo_list_bin.keys
		end

		def get_pkgset_list()
			return @pkgset_list.keys
		end

		def arch_available?(arch)
			return get_arch_list.include?(arch)
		end

		def get_arch_ignore(arch)
			return (@ignore_list.has_key?(arch))?@ignore_list[arch]:[]
		end

		def get_missing(arch)
			return nil if not arch_available?(arch)
			ign = get_ignore_list_match(arch)

			list = {}
			@pkgset_list.each do |name, pkgset|
				next if ign.include?(name)
				next if not pkgset.missing?(arch)
				listz = pkgset.get_missing_binaries(arch)
				list[name] = listz if not listz.empty?
			end

			return list
		end

		def get_all_missing()
			list = {}
			arch_list = get_arch_list()

			arch_list.each do |arch|
				list[arch] = get_missing(arch)
			end

			return list
		end

		def get_uncomplete(arch)
			return nil if not arch_available?(arch)
			ign = get_ignore_list_match(arch)

			list = {}
			@pkgset_list.each do |name, pkgset|
				next if ign.include?(name)
				next if not pkgset.uncomplete?(arch)
				next if pkgset.missing?(arch)
				listz = pkgset.get_missing_binaries(arch)
				if not @has_udeb
					listz_tmp = listz
					listz = []
					listz_tmp.each do |bin|
						listz<< bin if bin[-5..-1] != "-udeb"
					end
				end
				list[name] = listz if not listz.empty?
			end

			return list
		end

		def get_not_uptodate(arch, ign2)
			return nil if not arch_available?(arch)
			ign = get_ignore_list_match(arch)

			list = {}
			@pkgset_list.each do |name, pkgset|
				next if ign.include?(name)
				next if ign2.include?(name)
				next if pkgset.uncomplete?(arch)
				listz = pkgset.get_not_uptodate(arch)
				list[name] = listz if not listz.empty?
			end

			return list
		end

		def compute_ignore_list(arch)
			@ignore_list_match[arch] = []
			@ignore_list_bad[arch] = []

			@ignore_list[arch].each do |rx|
				begin
					regex = Regexp.new("^"+rx+"$", Regexp::IGNORECASE | Regexp::EXTENDED)
					used = false
					get_pkgset_list.each do |el|
						if el =~ regex
							@ignore_list_match[arch]<<  el
							used = true
						end
					end
					@ignore_list_bad[arch]<< rx if not used
				rescue
					@ignore_list_bad[arch]<< rx
				end
			end
			@ignore_list_match[arch].uniq!
		end

		def clear_ignore_list(arch)
			@ignore_list.delete(arch)
			@ignore_list_match.delete(arch)
			@ignore_list_bad.delete(arch)
		end

		def get_ignore_list(arch)
			return [] if not @ignore_list.has_key?(arch)
			return @ignore_list[arch]
		end

		def get_ignore_list_match(arch)
			return [] if not @ignore_list.has_key?(arch)
			compute_ignore_list(arch) if not @ignore_list_match.has_key?(arch) and not @ignore_list_bad.has_key?(arch)
			return @ignore_list_match[arch]
		end

		def get_ignore_list_bad(arch)
			return [] if not @ignore_list.has_key?(arch)
			compute_ignore_list(arch) if not @ignore_list_match.has_key?(arch) and not @ignore_list_bad.has_key?(arch)
			return @ignore_list_bad[arch]
		end

		def list_to_pkgsets(list)
			sets = []

			list.each do |name|
				sets<< @pkgset_list[name]
			end

			return sets
		end

		def split_by_priority(sets, do_sort=false)
			list = {}
			priority_lvl = PkgBin.priority_lvl

			priority_lvl.each_key do |prio|
				list[prio] = []
			end
			list[nil] = []

			sets.each do |pkgset|
				list[pkgset.get_pkgset_priority]<< pkgset
			end

			if do_sort
				priority_lvl.each_key do |prio|
					list[prio].sort!
				end
				list[nil].sort!
			end

			return list
		end

		def get_pkgset_count
			return @pkgset_list.length
		end

		def load_simple_list_from_file(filename, regex=nil)
			tab = []

			begin
				file = File.open(filename, "r")
				file.each_line do |line|
					line.chomp!
					tab<<line if (line != "") and ((not regex) or line =~ regex)
				end
				file.close()
				tab.sort!
			rescue
				STDERR.puts "Problems fetching '"+filename+"': "+$!
			end

			return tab
		end

		private :load_simple_list_from_file

	end

end

class HTML_RENDER
	def display_simple_tab(title, tab)
		str = "<div><span class='title1'>"+title+" ("+tab.length.to_s+") :</span><ul>"
		tab.each do |el|
			str+= "<li>"+yield(el)+"</li>"
		end
		str+= "</ul></div>"

		return str
	end

	def display_multicol_tab(title, tab, nb_cols=3)
		str = "<p>"
		str+= "<b><span class='title1'>"+title+" ("+tab.length.to_s+") :</span></b>"
		str+= "</p>"
		str+= "<table><tr><td>"
		i = 1
		nb_part = (tab.length.to_f / nb_cols.to_f).floor
		if (tab.class == "Hash")
			tab.each_key do |el|
				str+= yield(tab[el])+"<br />"
				if (i.to_f % nb_part) == 0
					str+= "</td><td>"
				end
				i = i.next
			end
		else
			tab.each do |el|
				str+= yield(el)+"<br />"
				if (i.to_f % nb_part) == 0
					str+= "</td><td>"
				end
				i = i.next
			end
		end
		str+= "</td></tr></table>"

		return str
	end
end

