
CREATE TABLE architectures (
	id_arch			INTEGER UNSIGNED AUTO_INCREMENT,
	architecture		CHAR(16),
				PRIMARY KEY (id_arch)
);

CREATE TABLE repositories (
	id_repo			INTEGER UNSIGNED AUTO_INCREMENT,
	repository		CHAR(256) NOT NULL,
				PRIMARY KEY (id_repo)
);

CREATE TABLE sections (
	id_section		INTEGER UNSIGNED AUTO_INCREMENT,
	section			CHAR(32) NOT NULL,
				PRIMARY KEY (id_section)
);

CREATE TABLE subsections (
	id_subsection		INTEGER UNSIGNED AUTO_INCREMENT,
	subsection		CHAR(32) NOT NULL,
				PRIMARY KEY (id_subsection)
);

CREATE TABLE priorities (
	id_prio			INTEGER UNSIGNED AUTO_INCREMENT,
	priority		CHAR(32) NOT NULL,
				PRIMARY KEY (id_prio)
);

CREATE TABLE pkg_src (
	id_pkg			INTEGER UNSIGNED AUTO_INCREMENT,
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL,
	maintainer		CHAR(128) NOT NULL,
	repository		INTEGER NOT NULL REFERENCES repositories(id_repo),
	obsolete		BOOLEAN NOT NULL DEFAULT false,
	format			CHAR(16) NOT NULL,
	standard_version	CHAR(16),
	directory		CHAR(256) NOT NULL,
				PRIMARY KEY (id_pkg)
);

CREATE TABLE src_architecture (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_src(id_pkg),
	id_arch			INTEGER NOT NULL REFERENCES architectures(id_arch)
);

CREATE TABLE src_build_depends (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_src(id_pkg),
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL
);

CREATE TABLE src_files (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_src(id_pkg),
	file			CHAR(256) NOT NULL
);

CREATE TABLE src_uploaders (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_src(id_pkg),
	uploader		CHAR(128) NOT NULL
);

CREATE TABLE src_binaries (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_src(id_pkg),
	package			CHAR(32) NOT NULL
);

CREATE TABLE pkg_bin (
	id_pkg			INTEGER UNSIGNED AUTO_INCREMENT,
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL,
	real_version		CHAR(32) NOT NULL,
	maintainer		CHAR(128) NOT NULL,
	repository		CHAR(256) NOT NULL,
	obsolete		BOOLEAN NOT NULL DEFAULT false,
	source			CHAR(32) NOT NULL,
	priority		INTEGER REFERENCES priorities(id_prio),
	section			INTEGER NOT NULL REFERENCES sections(id_section),
	subsection		INTEGER NOT NULL REFERENCES subsections(id_subsection),
	filename		CHAR(256) NOT NULL,
	size			INTEGER,
	installed_size		INTEGER,
	md5sum			CHAR(32) NOT NULL,
	short_description	CHAR(128) NOT NULL,
	description		CHAR(4096) NOT NULL,
				PRIMARY KEY (id_pkg)
);

CREATE TABLE bin_architecture (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_bin(id_pkg),
	id_arch			INTEGER NOT NULL REFERENCES architectures(id_arch)
);

CREATE TABLE bin_depends (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_bin(id_pkg),
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL
);

CREATE TABLE bin_suggests (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_bin(id_pkg),
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL
);

CREATE TABLE bin_recommends (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_bin(id_pkg),
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL
);

CREATE TABLE bin_conflicts (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_bin(id_pkg),
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL
);

CREATE TABLE bin_provides (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_bin(id_pkg),
	package			CHAR(32) NOT NULL
);

CREATE TABLE bin_replaces (
	id_pkg			INTEGER NOT NULL REFERENCES pkg_bin(id_pkg),
	package			CHAR(32) NOT NULL,
	version			CHAR(32) NOT NULL
);

